#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include "wbkernel.h"
#include "wbprezen.h"
#include "commands.h"
#include "wbcl.h"
#include "options.h"

static int handle_application(const char *appl)
{
	if (!status.connected){
	       	printf("Klient neni pripojen na server.\n");
		return 0;
	}
	while (isspace(*appl)) appl++;
	set_application(appl);
	return 0;
}

static int handle_resources(const char *)
{
	if (!status.connected){
	       	printf("Klient neni pripojen na server.\n");
		return 0;
	}
	printf("Pouzita pamet\n"
	       " Server   : %d\n"
	       " Klient   : %d\n",
		cd_Used_memory(cdp, 0), cd_Used_memory(cdp, 1));
	printf("Otevrene kurzory: %d\n", cd_Owned_cursors(cdp));
	return 0;
}

static int handle_show(const char *cmd)
{
	if (!status.connected){
	       	printf("Klient neni pripojen na server.\n");
		return 0;
	}
	unsigned size;
	while (isspace(*cmd)) cmd++;
	int type=Var_type_info(cdp, cmd, &size);
	switch (type){
	case 0:
		fprintf(stderr, "Promenna %s neni definovana v otevrenem projektu\n", cmd);
		return -1;
	case ATT_STRING:
		printf("%s\n", cd_Get_var_address(cdp, cmd));
		break;
	default:
		fprintf(stderr, "Typ %d\n");
	}
}

static int handle_speed_test(const char *)
{
	if (!status.connected){
	       	printf("Klient neni pripojen na server.\n");
		return 0;
	}
	int r1,r2;
	cd_Connection_speed_test(cdp, &r1, &r2);
	printf ("\nRychlost spojeni: %i, %i \n", r1, r2);
	return 0;
}

static int handle_status(const char *)
{
	if (!status.connected){
	       	printf("Klient neni pripojen na server.\n");
		return 0;
	}
	printf("Pripojen na server: %s\n", ServerName);
	if (!status.application_set)
		printf("Neni otevrena zadna aplikace\n");
	else
		printf("Aplikace: %s\n", Application);
	if (!status.logged)
		printf("Uzivatel neni prihlasen\n");
	else {
	  printf("Uzivatel prihlasen jako %s\n", cd_Who_am_I(cdp));
	  if (cd_Am_I_security_admin(cdp)) printf("Jsem bezpecnostni spravce\n");
	  if (cd_Am_I_config_admin(cdp)) printf("Jsem provozni spravce\n");
	  if (cd_Am_I_db_admin(cdp)) printf("Jsem datovy spravce\n");
	  if (status.application_set)
	    if (cd_Am_I_appl_admin(cdp)) printf("Jsem spravce aplikace\n");
	}
	return 0;
}

static int handle_project(const char *cmd)
{
	if (!status.connected){
	       	printf("Klient neni pripojen na server.\n");
		return 0;
	}
	while (isspace(*cmd)) cmd++;
	if (!cd_Open_project(cdp, cmd))
		write_error_info(cdp, "Open_project");
	return 0;
}

extern "C" int Exec_statements(cdp_t, const char *);
static int handle_ipj(const char *cmd)
{
	if (!status.connected){
	       	printf("Klient neni pripojen na server.\n");
		return 0;
	}
	int res = Exec_statements(cdp, cmd);
	if (res!=1)
		write_error_info(cdp, "IPJ", res);
	return 0;
}

static int handle_connect(const char *servername)
{
	while (isspace(*servername)) servername++;
	if (status.connected) cd_disconnect(cdp);
	status.logged=status.application_set=status.project_open=false;
	int res=cd_connect(cdp, servername, 0);
	if (res) write_error_info(cdp, "connect", res);
	else strmaxcpy(ServerName, servername, sizeof(ServerName));
	status.connected=!res;
	return 0;
}

static int handle_login(const char *cmd)
{
	char *pwd;
	char prompt[256];
	if (!status.connected)
	{ fprintf(stderr, "Pred prihlasenim je nutno se pripojit\n");
	  return 0;
	}
	while (isspace(*cmd)) cmd++;
	snprintf(prompt, sizeof(prompt), "Heslo pro %s: ", cmd);
	pwd=getpass(prompt);
	int res=cd_Login(cdp, cmd, pwd);
	if (res)
		write_error_info(cdp, "Login", res);
	else strmaxcpy(LoginName, cmd, sizeof(LoginName));
	status.logged=!res;
	return 0;
}

static int handle_servers(const char *cmd)
{
    printf("Viditelne servery: \n");
    enum_servers(200, NULL, NULL);
    return 0;
}

static int handle_help(const char *topic)
{
  struct command_t *cmd=commands_list;
  while (isspace(*topic)) topic++;
  while (cmd->prefix!=NULL){
	  size_t len=strlen(cmd->prefix);
	  if (0==*topic ||0==strncmp(cmd->prefix, topic , len))
		  printf("%s\n", (const char *)cmd->help);
	  cmd++;
  }
  return 0;
}

/* Table of built-in commands */
struct command_t commands_list[]= {
	{"help", handle_help, {
		"help [<command>] --- show help",
		"help [<prikaz>] --- zobraz napovedu"}},
	{"connect", handle_connect, {
		"connect <server> --- connect to new server",
		"connect <server> --- pripoj se k serveru"}},
	{"login", handle_login, {
		"login <name> [<password>] --- login as a new user",
		"login <jmeno> [<heslo>] --- proved prilogovani"}},
	{"servers", handle_servers, {
		"servers --- list available servers",
		"servers --- vypis seznam serveru"}},
	{"ipj", handle_ipj, {
		"ipj <command> --- execute command in client language",
		"ipj <prikaz> --- proved prikaz klientskeho jazyka"}},
	{"project", handle_project,{
		"project <projectname> --- open client project",
		"project <jmeno projektu> --- otevri projekt" }},
	{"status", handle_status,{
		"status --- show whether connected and logged, application and project",
		"status ---- zobraz parametry soucasneho pripojeni"}},
	{"application", handle_application,{
		"application <applname> --- change application",
	   	"application <applname> --- zmenit aplikaci"}},
	{"resources", handle_resources, {
		"resources --- show memory usage",
		"resources --- zobraz spotrebu pameti"
	}},
	{"speed", handle_speed_test,{
		"speed  --- test speed of communication" ,
		"speed  --- otestuj rychlost komunikace"}
	},
	{"show", handle_show,{
		"show <variable> --- show value of variable",
		"show <variable> --- zobraz hodnotu promenne"}
	},
	{NULL, NULL, {NULL, NULL}}
};

int multil_text::language = 1;

