/* $Id: do_sql.cpp,v 1.9 2001/08/10 10:50:29 zellerin Exp $
 * (Do_SQL) Read one or more SQL command(s) from command line and execute.
 */

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <obstack.h>
#include <malloc.h>
#include <ctype.h>

#include <wbkernel.h>
#include "wbcl.h"
#include "attrib.h"
#include "options.h"
#include "macros.h"
#include "commands.h"
#include <argz.h>
#include <envz.h>


/* Some versions of readline library have wrong declaration. */
extern "C" void add_history(const char *);

/* Po provedeni update, insert  a pod. se vypise pocet provedenych zaznamu */
static inline void result_count(int count)
{
    if (verbose)
	    printf("pocet zpracovanych zaznamu nebo cislo objektu: %d \n", count);
}

/* Navratova hodnota SQL_execute muze byt bud pocet zpracovanych zaznamu nebo
 * kurzor. */ 
static inline void display_result(uns32 result)
{
    if (result==(uns32)-1) return;
    if (IS_CURSOR_NUM(result)) Display_Cursor(cdp, result);
    else result_count(result);
}

/* Obstack library needs those declarations. */
#define obstack_chunk_alloc malloc
#define obstack_chunk_free free

/* Longer commands are stored inside */
static struct obstack obst;

/* Table of commands expanded to sql. They are read from config file */
struct command_t *user_commands=NULL;

static int do_one_general_command(const char *command)
{
  struct command_t *cmd=commands_list;
#ifdef USE_READLINE
    add_history(command);
#endif
#define strdupa strdup
    const char *expansion=expand_command(strdupa(command));
    if (expansion) command=expansion;
    while (cmd->prefix!=NULL){
      size_t len=strlen(cmd->prefix);
      if (0==strncmp(cmd->prefix, command , len))
	      return cmd->action(strdupa(command+len));
      cmd++;
    }
    return do_one_command(command);
}

/* Execute one SQL command and print results. */
int do_one_command(const char *SQLexecute)
{
    uns32 result[10];
    for (uns32 *res=result; res<result+10; res++) *res=(uns32)-1;
    result[0]=0;
    if (verbose)
	    printf("\nZpracovavam SQL prikaz \n%s\n", SQLexecute);
    if (cd_SQL_execute(cdp, SQLexecute, result)) 
	    write_error_info(cdp, "SQL_execute ");
    else
	   for (uns32*res=result; res<result+10; res++)
		   display_result(*res);
    return 0;
}

/* Read one line from the stdin and, depending on the value of the
 * text_may_span, or execute, or append to the obstack */

extern "C" char * readline(char *);
inline char * get_line()
{
	static char prompt[]="602cli8: ";
#ifdef USE_READLINE
    if (interactive) return readline(prompt);
#else
    if (interactive) 
	    printf("%s",prompt);
#endif
    char * str=(char *) malloc(1024);
    if (NULL==fgets(str, 1023, stdin)){
	    free(str);
	    return NULL;
    }
    char *ptr=strchr(str, '\n');
    if (ptr) *ptr='\0';
    return str;
}

static int Do_one_line(void)
{
    char *SQLexecute;
    if (verbose&&!text_may_span)
	    printf("\nSQL prikaz nebo interni prikaz (help=seznam) nebo q (=quit): \n");
    if ((SQLexecute=get_line())==NULL || 0==strcmp(SQLexecute, "q"))
	    return FALSE;
    if (text_may_span){
	    obstack_grow(&obst, SQLexecute, strlen(SQLexecute));
	    obstack_grow(&obst, " ", 1);
    }  else {
	    do_one_general_command(SQLexecute);
    }
    free(SQLexecute);
    return TRUE;
}

/* Read lines from stdin and execute them or sequentialy, or all in once. */
int Do_SQL()
{
    if (text_may_span){
	    obstack_init(&obst);
	    if (verbose)
	    printf("\nSQL prikaz ukonceny q: \n");
    }
    while(Do_one_line());
    if (text_may_span){
	    char *sql_text=(char *)obstack_finish(&obst);
	    do_one_general_command(sql_text);
	    obstack_free(&obst, sql_text);
    }
    return 0;
}
