/*
 * (read_from_ini) read values of options from resources
 * (read_from_argv) ditto from command line arguments
 */

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <getopt.h>
#include <stdlib.h>
#include <wbkernel.h>
#include <wordexp.h>
#include <assert.h>

#include "wbcl.h"
#include "options.h"

// globalni promene :
bool interactive=isatty(0);    // stdin from terminal?
bool disp_attrname=0;  //zobrazuj jmena atributu
bool disp_attrtype=0;  //zobrazuj typ atributu
char disp_recsep[8]; //record separator
char disp_attrsep[8];//attr separator
bool disp_recnum=0;    //zobrazuj cislo zaznamu pred kazdym zaznamem
bool verbose=interactive;        //ukecany mod
bool disp_descr=0;     //zobrazuj popis u nezobrazitelnych atributu
bool text_may_span=0;
int disp_textl;      //zobrazuj n bytes textoveho atributu, max 64kb
int disp_mode=!verbose;       //mod zobrazeni : 0 jeden zaznam, jeden atribut na radku
                     //                1 jeden zaznam na jeden radek
/* Default values of login parameters */
char ServerName [OBJ_NAME_LEN+1]="";
char LoginName  [OBJ_NAME_LEN+1]="";
char Password[50]="";
char Application[OBJ_NAME_LEN+1]="";
// import parameters:
char import_file[256] = "";
int  import_flags = 0;
char alt_name   [OBJ_NAME_LEN+1]="";

struct var_desc_impl:public var_desc {
	inline var_desc_impl(const char *key, char *location, int size) :
	  var_desc(key, vt_str, location, size) {}
	inline var_desc_impl(const char *key, bool *location) :
	  var_desc(key, vt_bool, location) {}
	inline var_desc_impl(const char *key, int *location) :
	  var_desc(key, vt_int, location) {}
};

var_desc var_desc::internal_variables[]={
	var_desc_impl("ServerName",  ServerName, sizeof(ServerName)),
	var_desc_impl("LoginName",  LoginName, sizeof(LoginName)),
	var_desc_impl("Password",  Password, sizeof(Password)),
	var_desc_impl("Application",  Application, sizeof(Application)),
	var_desc_impl("Verbose",  &verbose),
	var_desc_impl("DispAttrName",  &disp_attrname),
	var_desc_impl("DispAttrType",  &disp_attrtype),
	var_desc_impl("DispRecNum",  &disp_recnum),
	var_desc_impl("DispDescr",  &disp_descr),
	var_desc_impl("DispTextLen",  &disp_textl),
	var_desc_impl("DispMode",  &disp_mode)
};

int var_desc::internal_variables_count=
	sizeof(var_desc::internal_variables)/sizeof (var_desc_impl);

static const char clientini[]="602client"; /* jmeno sekce, kde budu hledat parametry */

static void string_var(const char *key, char *buff, size_t bufsize, const char *dfl="")
{
	GetPrivateProfileString(clientini,key,dfl,buff,bufsize,configfile);
}

static void bool_var(const char *key, int*res, int dfl=0)
{
	char tmp[50];
	GetPrivateProfileString(clientini, key,
		dfl?"true":"false",tmp,sizeof(tmp),configfile);
	if (strncasecmp(tmp,"false",4) == 0
		|| strncmp(tmp, "0", 2)==0 ) *res=0;
	else *res=1;
}

static void int_var(const char *key, int *res, int dfl=0)
{
	*res=GetPrivateProfileInt(clientini, key, dfl, configfile);
}

int read_from_ini()
// Loads initial values of [internal_variables] from [configfile]
{
	var_desc *desc=var_desc::internal_variables,
		*descend=var_desc::internal_variables
			+var_desc::internal_variables_count;
	while (desc<descend){
		switch(desc->type){
		case vt_int:
		       	int_var(desc->key, (int *)desc->location, *(const int *)desc->location);
			break;
		case vt_bool:
			bool_var(desc->key, (int *)desc->location, *(const int *)desc->location);
			break;
		case vt_str:
			string_var(desc->key, (char *)desc->location, desc->size, (const char *)desc->location);
		}
		desc++;
	}
	return 0;
}



/*
 * Texts to be printed
 */
static char wbHdr[]      = "\n602sql Client\n";
static char napoveda[]   = "\n"
"602cli8 [-v -dn -dt -dd -dr -dl<n> -dm<n> -s<> -l<> -p<> -a<> -cLQ]\n\
-v  verbose                      -s<server_name>\n\
-dn display attr. name           -l<login_name>\n\
-dt               type           -p<password>\n\
-dd               description    -a<application_name>\n\
-dr         record number\n\
-dl<n> display <n> bytes of text attr. ( 0 - only descr.)\n\
-dm<n> display mode : 0 - one record per page\n\
                      1 - all records and one record per line\n"
"-L   long (multiline) commands\n"
"-Q Query for servers on local net\n";

void read_from_argv(int argc, char **argv)
{
  int opt;
  while ((opt=getopt(argc, argv, "qchd:vs:l:p:a:LQi:I:A:"))!=-1)
  switch (opt){
  case 'q':
	  verbose=0;
	  disp_attrname=0;
	  disp_attrtype=0;
	  disp_recnum=0;
	  disp_descr=0;
	  break;
  case  'L':
    text_may_span=1;
    break;
  case  'v':
    verbose=true;
    break;
  case 'd':
    while (*optarg!='\0')
    switch (*(optarg++)){
#define SETOPTARG(letter, var) \
    case letter:\
     if (*optarg!='-') var = TRUE;\
     else {\
       var = FALSE;\
       optarg++;\
     }\
     break;

    SETOPTARG('n', disp_attrname);
    SETOPTARG('t', disp_attrtype);
    SETOPTARG('r', disp_recnum);
    SETOPTARG('d', disp_descr);
    case 'l':
      optarg+=sscanf(optarg, "%d", &disp_textl);
      break;
    case 'm':
      optarg+=sscanf(optarg, "%d", &disp_mode);
      break;
    default:
      printf(wbHdr);
      printf(napoveda);
      exit(EXIT_FAILURE);
    }
    break;
#undef SETOPTARG
  case 's':
    strmaxcpy(ServerName,  optarg, sizeof(ServerName));
    break;
  case 'l':
    strmaxcpy(LoginName,   optarg, sizeof(LoginName));
    break;
  case 'p':
    strmaxcpy(Password,    optarg, sizeof(Password));
    break;
  case 'a':
    strmaxcpy(Application, optarg, sizeof(Application));
    break;
  case 'Q':
    enum_servers(200, NULL, NULL);  // timeout=200
    exit(0);

 // import parameters:
  case 'i':  // import file name
    strmaxcpy(import_file, optarg, sizeof(import_file));
    break;
  case 'I':  // import flags
    import_flags = atoi(optarg);
    break;
  case 'A':  // alternate name for the imported application
    strmaxcpy(alt_name, optarg, sizeof(alt_name));
    break;

  case '?': case 'h':
    printf(wbHdr);
    printf(napoveda);
    exit(1);
  }
}

/* Nastaveni parametru. Vraci pokud v poradku (parametr nastaven nebo
 * existuje fallback hodnota), konci pokud nelze pokracovat. Krome toho vypise
 * varovani pokud parametr neni nastaven.
 */
static void check_param(const char *name, const char *val, const char * fallback)
{
    if (*val!='\0') return;
    if (fallback==NULL){
	    fprintf(stderr, "Nenasel se parametr %s - koncim\n", name);
	    exit(-1);
    }
    if (verbose) fprintf(stderr, "Nenasel se parametr %s, pouziji %s\n", name, fallback);
}

void check_params()
{
    check_param("ServerName", ServerName, NULL);
    check_param("LoginName", LoginName, "Anonymous");
    check_param("Password", Password, "prazdne heslo");
    if (!import_file[0])
      check_param("Application", Application, NULL);
}
