/************************************************************************
 * wbcl.cpp - klient pro Unix v prikazove radce                         *
 * $date:$								*
 ************************************************************************/
/* system headers */
#define _GNU_SOURCE
#include <signal.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include "macros.h"

extern "C" int write_history(const char *);
extern "C" int read_history(const char *);

/* WinBase headers */
#include "wbkernel.h"
#include "wbprezen.h"

/* wbcl headers */
#include "options.h"
#include "attrib.h"
#include "wbcl.h"

/* Kontextov promnn klienta */
cdp_t cdp;

/* Fatalni chyba. Vypis problem a zmiz. */
static void __attribute__((noreturn)) fatal(const char * descr, int errnum)
{
	char errtext[255];
	
	Get_error_num_text(cdp, errnum, errtext, sizeof(errtext));
	printf("%s: %s\n", descr, errtext);
	exit (EXIT_FAILURE);
}

static char *history_file;
static int no_history;

status_t status;
#ifdef USE_READLINE
extern inline int store_history()
{
	if (!no_history) write_history(history_file);
	return 0;
}
#else
extern inline int store_history(){}
#endif


/* Funkce pro zpracovn signlu koncho program. */
static void TermAbortProc(int sig)
{
	static volatile sig_atomic_t fatal_error_in_progress = 0;

	if (fatal_error_in_progress)
		raise (sig);

	fatal_error_in_progress=1;
	psignal(sig, "602sqlclient");
	cd_disconnect(cdp);
	store_history();
	signal(sig, SIG_DFL);
	raise(sig);
}

/* Set application to the chosen one. Handle possible problem in one of
 * possible ways */

void set_application(const char *appl)
{
	int res;
	res=cd_Set_application(cdp, appl);
	if (!res){
	       	status.application_set=(0!=*appl);
		if (appl!=Application)
			strmaxcpy(Application, appl, sizeof(Application));
	} else write_error_info(cdp, "set application");
}

int import_nad_exit(void)
// Import application specified on the command line and exit
{	cd_t my_cd;  cdp=&my_cd;  int res;
	res=cd_connect(cdp, ServerName, 0);
	if (res!=0)
		write_error_info(cdp, "connect", res);
	else
	{	res=cd_Login(cdp, LoginName, Password);
		if (res)
			write_error_info(cdp, "Login");
		else
			res=Import_appl_ex(cdp, 0, import_file, import_flags, alt_name) ? 0 : 10000;
		cd_disconnect(cdp);
	}
	return res;
}

int main(int argc, char *argv[])
{
	signal(SIGTERM, TermAbortProc);
	signal(SIGABRT, TermAbortProc);
	signal(SIGINT, TermAbortProc);

	read_from_ini();
	read_from_argv(argc, argv);
	check_params();
	if (import_file[0]) return import_nad_exit();
#ifdef USE_READLINE
	const char *home=getenv("HOME");
	asprintf(&history_file, "%s/.wbcl_history", home);
	no_history=read_history(history_file);
#endif

	cd_t my_cd; /* Nastav kontextovou promennou */
	cdp=&my_cd;

	load_macros("602client.rc", "macros");
	load_macros("/usr/share/602sql/602client", "macros");
	/* Pripojeni k serveru */
	/* Inicializace spojeni mezi klientem a serverem */
	int res=cd_connect(cdp, ServerName, 0);
	status.connected=!res;
	if (res!=0) write_error_info(cdp, "connect", res);
	else if (res==0 && verbose) printf("(connected)\n");

	/* Pokusme se zalogovat */
	if (status.connected){
		res=cd_Login(cdp, LoginName, Password);
		if (res) write_error_info(cdp, "Login");
		status.logged=!res;
		status.application_set=false;
		if (status.logged) set_application(Application);
	}
	Do_SQL();
	if (status.logged) cd_Logout(cdp);
	status.logged=false;
	if (status.connected) cd_disconnect(cdp);
	status.connected=false;
	store_history();
	if (verbose) printf("\n602cli8 ukoncen.\n");
	return 0;
}


void write_error_info(cdp_t cdp, const char * caller, int err)
{
	char buffer[256];
	Get_error_num_text(cdp, err, buffer, sizeof(buffer));
	printf("\nChyba %s (# %d) v %s.\n", buffer, err, caller);
}

const char *configfile="/etc/602sql";

void strmaxcpy(char * dest, const char * src, size_t size)
{ int len=strlen(src);
  if (len>=size)
  { if (!size) return;
    len=size-1;
  }
  memcpy(dest, src, len);
  dest[len]=0;
}

